@@ -1,7 +1,8 @@
 local util      = require 'utility'
 local guide     = require 'parser.guide'
-local collector = require 'core.collector'
+local collector = require 'core.collector' 'searcher'
 local files     = require 'files'
+local config    = require 'config'
 
 local tostring = tostring
 local error    = error
@@ -26,7 +27,6 @@ local ANY_FIELD_CHAR = '*'
 local INDEX_CHAR     = '['
 local RETURN_INDEX   = SPLIT_CHAR .. '#'
 local PARAM_INDEX    = SPLIT_CHAR .. '&'
-local PARAM_NAME     = SPLIT_CHAR .. '$'
 local EVENT_ENUM     = SPLIT_CHAR .. '>'
 local TABLE_KEY      = SPLIT_CHAR .. '<'
 local WEAK_TABLE_KEY = SPLIT_CHAR .. '<<'
@@ -94,6 +94,10 @@ local INFO_DEEP_AND_DONT_CROSS = {
 ---@field binfo?    table<node.id, node.info>
 -- 后退的关联ID与info
 ---@field backwards table<node.id, node.id[]|table<node.id, node.info>>
+-- 第一个继承
+---@field extend    table<node.id, node.id>
+-- 其他继承
+---@field extends   table<node.id, node.id[]>
 -- 函数调用参数信息（用于泛型）
 ---@field call      table<node.id, parser.guide.object>
 ---@field require   table<node.id, string>
@@ -149,11 +153,23 @@ local function getFieldEventName(field)
     if not docFunc or docFunc.type ~= 'doc.type.function' then
         return nil
     end
-    local firstArg = docFunc.args and #docFunc.args == 2 and docFunc.args[1]
+    local firstArg = docFunc.args and docFunc.args[1]
     if not firstArg then
         return nil
     end
-    local secondArg = docFunc.args[2]
+    local secondArg
+    if firstArg.name[1] == 'self' then
+        firstArg = docFunc.args[2]
+        if not firstArg then
+            return nil
+        end
+        secondArg = docFunc.args[3]
+    else
+        secondArg = docFunc.args[2]
+    end
+    if not secondArg then
+        return
+    end
     local firstType = firstArg.extends
     if not firstType then
         return nil
@@ -481,9 +497,11 @@ local function getNodeKey(source)
     if methodNode then
         return getNodeKey(methodNode)
     end
-    local localValueID = getLocalValueID(source)
-    if localValueID then
-        return localValueID
+    if config.get(guide.getUri(source), 'Lua.IntelliSense.traceFieldInject') then
+        local localValueID = getLocalValueID(source)
+        if localValueID then
+            return localValueID
+        end
     end
     local key, node = getKey(source)
     if key and guide.isGlobal(source) then
@@ -534,8 +552,8 @@ end
 
 ---添加关联的前进ID
 ---@param noders    noders
----@param id        string
----@param forwardID string
+---@param id        node.id
+---@param forwardID node.id
 ---@param info?     node.info
 local function pushForward(noders, id, forwardID, info)
     if not id
@@ -566,8 +584,8 @@ end
 
 ---添加关联的后退ID
 ---@param noders     noders
----@param id         string
----@param backwardID string
+---@param id         node.id
+---@param backwardID node.id
 ---@param info?      node.info
 local function pushBackward(noders, id, backwardID, info)
     if not id
@@ -596,6 +614,36 @@ local function pushBackward(noders, id, backwardID, info)
     backwards[#backwards+1] = backwardID
 end
 
+---添加继承的关联ID
+---@param noders     noders
+---@param id         node.id
+---@param extendID   node.id
+local function pushExtend(noders, id, extendID)
+    if not id
+    or not extendID
+    or extendID == ''
+    or id == extendID then
+        return
+    end
+    if not noders.extend[id] then
+        noders.extend[id] = extendID
+        return
+    end
+    if noders.extend[id] == extendID then
+        return
+    end
+    local extends = noders.extends[id]
+    if not extends then
+        extends = {}
+        noders.extends[id] = extends
+    end
+    if extends[extendID] ~= nil then
+        return
+    end
+    extends[extendID] = false
+    extends[#extends+1] = extendID
+end
+
 ---@class noder
 local m = {}
 
@@ -736,6 +784,31 @@ function m.eachBackward(noders, id)
     end
 end
 
+---遍历extend
+---@param noders noders
+---@param id node.id
+---@return fun():string, node.info
+function m.eachExtend(noders, id)
+    local extend = noders.extend[id]
+    if not extend then
+        return DUMMY_FUNCTION
+    end
+    local index
+    local extends = noders.extends[id]
+    return function ()
+        if not index then
+            index = 0
+            return extend
+        end
+        if not extends then
+            return nil
+        end
+        index = index + 1
+        local id  = extends[index]
+        return id
+    end
+end
+
 local function bindValue(noders, source, id)
     local value = source.value
     if not value then
@@ -745,12 +818,17 @@ local function bindValue(noders, source, id)
     if not valueID then
         return
     end
+
+    local bindDocs = source.bindDocs
     if source.type == 'getlocal'
     or source.type == 'setlocal' then
-        source = source.node
+        if not config.get(guide.getUri(source), 'Lua.IntelliSense.traceLocalSet') then
+            return
+        end
+        bindDocs = source.node.bindDocs
     end
-    if source.bindDocs and value.type ~= 'table' then
-        for _, doc in ipairs(source.bindDocs) do
+    if bindDocs and value.type ~= 'table' then
+        for _, doc in ipairs(bindDocs) do
             if doc.type == 'doc.class'
             or doc.type == 'doc.type' then
                 return
@@ -759,6 +837,10 @@ local function bindValue(noders, source, id)
     end
     -- x = y : x -> y
     pushForward(noders, id, valueID, INFO_REJECT_SET)
+    if  not config.get(guide.getUri(source), 'Lua.IntelliSense.traceBeSetted')
+    and source.type ~= 'local' then
+        return
+    end
     -- 参数/call禁止反向查找赋值
     local valueType = smatch(valueID, '^(.-:).')
     if not valueType then
@@ -787,8 +869,10 @@ local function compileCallParam(noders, call, sourceID)
     if not nodeID then
         return
     end
+    local methodIndex = 0
     if node.type == 'getmethod' then
         fixIndex = fixIndex + 1
+        methodIndex = 1
     end
     local eventNodeID
     for firstIndex, callArg in ipairs(call.args) do
@@ -808,7 +892,7 @@ local function compileCallParam(noders, call, sourceID)
                     local paramID = sformat('%s%s%s%s%s'
                         , nodeID
                         , PARAM_INDEX
-                        , firstIndex
+                        , firstIndex + methodIndex
                         , PARAM_INDEX
                         , secondIndex
                     )
@@ -817,7 +901,7 @@ local function compileCallParam(noders, call, sourceID)
                         local eventParamID = sformat('%s%s%s%s%s'
                             , eventNodeID
                             , PARAM_INDEX
-                            , firstIndex
+                            , firstIndex + methodIndex
                             , PARAM_INDEX
                             , secondIndex
                         )
@@ -826,6 +910,14 @@ local function compileCallParam(noders, call, sourceID)
                 end
             end
         end
+        if callArg.type == 'table' then
+            local paramID = sformat('%s%s%s'
+                , nodeID
+                , PARAM_INDEX
+                , firstIndex + methodIndex
+            )
+            pushForward(noders, getID(callArg), paramID)
+        end
     end
 end
 
@@ -924,33 +1016,14 @@ compileNodeMap = util.switch()
     : call(function (noders, id, source)
         pushForward(noders, id, 'dn:nil')
     end)
-    : case 'local'
-    : call(function (noders, id, source)
-        if source[1] ~= 'self' then
-            return
-        end
-        local func = guide.getParentFunction(source)
-        if func.isGeneric then
-            return
-        end
-        if source.parent.type ~= 'funcargs' then
-            return
-        end
-        local setmethod = func.parent
-        -- guess `self`
-        if setmethod and ( setmethod.type == 'setmethod'
-                        or setmethod.type == 'setfield'
-                        or setmethod.type == 'setindex') then
-            pushForward(noders, id, getID(setmethod.node))
-            pushBackward(noders, getID(setmethod.node), id, INFO_DEEP)
-        end
-    end)
     : case 'doc.type'
     : call(function (noders, id, source)
         if source.bindSources then
             for _, src in ipairs(source.bindSources) do
-                pushForward(noders, getID(src), id)
-                pushBackward(noders, id, getID(src))
+                if  src.parent.type ~= 'funcargs'
+                and not src.dummy then
+                    pushForward(noders, getID(src), id)
+                end
             end
         end
         for _, enumUnit in ipairs(source.enums) do
@@ -964,7 +1037,10 @@ compileNodeMap = util.switch()
             pushForward(noders, id, unitID)
             if source.bindSources then
                 for _, src in ipairs(source.bindSources) do
-                    pushBackward(noders, unitID, getID(src))
+                    if  src.parent.type ~= 'funcargs'
+                    and not src.dummy then
+                        pushBackward(noders, unitID, getID(src))
+                    end
                 end
             end
         end
@@ -1032,13 +1108,17 @@ compileNodeMap = util.switch()
         pushForward(noders, getID(source.class), id)
         if source.extends then
             for _, ext in ipairs(source.extends) do
-                pushForward(noders, id, getID(ext), INFO_CLASS_TO_EXNTENDS)
+                pushExtend(noders, id, getID(ext))
             end
         end
         if source.bindSources then
             for _, src in ipairs(source.bindSources) do
-                pushForward(noders, getID(src), id)
-                pushForward(noders, id, getID(src))
+                if  src.parent.type ~= 'funcargs'
+                and src.type ~= 'setmethod'
+                and not src.dummy then
+                    pushForward(noders, getID(src), id)
+                    pushForward(noders, id, getID(src))
+                end
             end
         end
         for _, field in ipairs(source.fields) do
@@ -1069,7 +1149,6 @@ compileNodeMap = util.switch()
                 pushForward(noders, keyID, getID(field.field))
                 pushForward(noders, getID(field.field), keyID)
                 pushForward(noders, keyID, getID(field.extends))
-                pushBackward(noders, getID(field.extends), keyID)
             end
         end
     end)
@@ -1097,14 +1176,12 @@ compileNodeMap = util.switch()
         end
         if source.bindSources then
             for _, src in ipairs(source.bindSources) do
-                if src.type == 'function'
-                or guide.isSet(src) then
-                    local paramID = sformat('%s%s%s'
-                        , getID(src)
-                        , PARAM_NAME
-                        , source.param[1]
-                    )
-                    pushForward(noders, paramID, id)
+                if src.type == 'function' and src.args then
+                    for _, arg in ipairs(src.args) do
+                        if arg[1] == source.param[1] then
+                            pushForward(noders, getID(arg), id)
+                        end
+                    end
                 end
             end
         end
@@ -1145,12 +1222,6 @@ compileNodeMap = util.switch()
     : call(function (noders, id, source)
         if source.args then
             for index, param in ipairs(source.args) do
-                local paramID = sformat('%s%s%s'
-                    , id
-                    , PARAM_NAME
-                    , param.name[1]
-                )
-                pushForward(noders, paramID, getID(param.extends))
                 local indexID = sformat('%s%s%s'
                     , id
                     , PARAM_INDEX
@@ -1183,59 +1254,19 @@ compileNodeMap = util.switch()
     : case 'doc.type.name'
     : call(function (noders, id, source)
         local uri = guide.getUri(source)
-        collector.subscribe(uri, id, noders)
-
-        local parent = source.parent
-        if parent.type ~= 'doc.type' then
-            goto BREAK
-        end
-        local bindSources = parent.bindSources
-        if not bindSources then
-            goto BREAK
-        end
-        for i = 1, #bindSources do
-            local src = bindSources[i]
-            if src.type ~= 'local' then
-                goto CONTINUE1
-            end
-            local refs = src.ref
-            if not refs then
-                goto CONTINUE1
-            end
-            for j = 1, #refs do
-                local ref = refs[j]
-                if ref.type ~= 'getlocal' then
-                    goto CONTINUE2
-                end
-                local nxt = ref.next
-                if not nxt then
-                    goto CONTINUE2
-                end
-                local nxtType = nxt.type
-                if nxtType == 'setfield'
-                or nxtType == 'setmethod'
-                or nxtType == 'setindex' then
-                    local defID = 'def:' .. id
-                    collector.subscribe(uri, defID, noders)
-                    goto BREAK
-                end
-                ::CONTINUE2::
-            end
-            ::CONTINUE1::
-        end
-        ::BREAK::
+        collector:subscribe(uri, id, noders)
     end)
     : case 'doc.class.name'
     : case 'doc.alias.name'
     : call(function (noders, id, source)
         local uri = guide.getUri(source)
-        collector.subscribe(uri, id, noders)
+        collector:subscribe(uri, id, noders)
 
         local defID = 'def:' .. id
-        collector.subscribe(uri, defID, noders)
+        collector:subscribe(uri, defID, noders)
 
         local defAnyID = 'def:dn:'
-        collector.subscribe(uri, defAnyID, noders)
+        collector:subscribe(uri, defAnyID, noders)
     end)
     : case 'function'
     : call(function (noders, id, source)
@@ -1276,31 +1307,13 @@ compileNodeMap = util.switch()
                     , i
                 )
                 pushForward(noders, indexID, getID(arg))
-                if arg.type == 'local' then
-                    pushForward(noders, getID(arg), sformat('%s%s%s'
-                        , id
-                        , PARAM_NAME
-                        , arg[1]
-                    ))
-                    if parentID then
-                        pushForward(noders, getID(arg), sformat('%s%s%s'
-                            , parentID
-                            , PARAM_NAME
-                            , arg[1]
-                        ))
-                    end
-                else
-                    pushForward(noders, getID(arg), sformat('%s%s%s'
-                        , id
-                        , PARAM_NAME
-                        , '...'
-                    ))
-                    if parentID then
-                        pushForward(noders, getID(arg), sformat('%s%s%s'
-                            , parentID
-                            , PARAM_NAME
-                            , '...'
-                        ))
+                if arg.type ~= 'local' then
+                    for j = i + 1, i + 10 do
+                        pushForward(noders, sformat('%s%s%s'
+                            , id
+                            , PARAM_INDEX
+                            , j
+                        ), getID(arg))
                     end
                 end
                 ::CONTINUE::
@@ -1316,7 +1329,9 @@ compileNodeMap = util.switch()
                         , index
                     )
                     pushForward(noders, returnID, getID(rtnObj))
-                    pushBackward(noders, getID(rtnObj), returnID, INFO_DEEP_AND_DONT_CROSS)
+                    if config.get(guide.getUri(source), 'Lua.IntelliSense.traceReturn') then
+                        pushBackward(noders, getID(rtnObj), returnID, INFO_DEEP_AND_DONT_CROSS)
+                    end
                 end
             end
         end
@@ -1346,6 +1361,17 @@ compileNodeMap = util.switch()
                 end
             end
         end
+        local parent = source.parent
+        if guide.isSet(parent)  then
+            pushForward(noders, id, getID(parent))
+        end
+    end)
+    : case 'loop'
+    : call(function (noders, id, source)
+        local loc = source.loc
+        if loc then
+            pushForward(noders, getID(loc), 'dn:integer')
+        end
     end)
     : case 'in'
     : call(function (noders, id, source)
@@ -1416,8 +1442,7 @@ compileNodeMap = util.switch()
         end
         for _, rtn in ipairs(source.returns) do
             for _, src in ipairs(source.bindSources) do
-                if src.type == 'function'
-                or guide.isSet(src) then
+                if src.type == 'function' then
                     local fullID = sformat('%s%s%s'
                         , getID(src)
                         , RETURN_INDEX
@@ -1453,7 +1478,6 @@ compileNodeMap = util.switch()
             if upvalues[key] then
                 for _, paramID in ipairs(upvalues[key]) do
                     pushForward(noders, id, paramID)
-                    pushBackward(noders, paramID, id)
                 end
             end
         end
@@ -1487,18 +1511,24 @@ function m.compileNode(noders, source)
 
     if id and ssub(id, 1, 2) == 'g:' then
         local uri = guide.getUri(source)
-        collector.subscribe(uri, id, noders)
+        collector:subscribe(uri, id, noders)
         if  guide.isSet(source)
         -- local t = Global --> t: g:.Global
         and source.type ~= 'local'
         and source.type ~= 'setlocal' then
 
             local defID = 'def:' .. id
-            collector.subscribe(uri, defID, noders)
+            collector:subscribe(uri, defID, noders)
+
+            local fieldID = m.getLastID(id)
+            if fieldID then
+                local defNodeID = 'field:' .. fieldID
+                collector:subscribe(uri, defNodeID, noders)
+            end
 
             if guide.isGlobal(source) then
                 local defAnyID = 'def:g:'
-                collector.subscribe(uri, defAnyID, noders)
+                collector:subscribe(uri, defAnyID, noders)
             end
         end
     end
@@ -1546,6 +1576,11 @@ function m.getLastID(id)
     return lastID
 end
 
+function m.getFieldID(id)
+    local fieldID = smatch(id, LAST_REGEX)
+    return fieldID
+end
+
 ---获取ID的长度
 ---@param id string
 ---@return integer
@@ -1571,7 +1606,7 @@ function m.hasField(id)
     end
     local next2Char = ssub(id, #firstID + 2, #firstID + 2)
     if next2Char == RETURN_INDEX
-    or next2Char == PARAM_NAME then
+    or next2Char == PARAM_INDEX then
         return false
     end
     return true
@@ -1595,9 +1630,6 @@ function m.isCommonField(field)
     if ssub(field, 1, #RETURN_INDEX) == RETURN_INDEX then
         return false
     end
-    if ssub(field, 1, #PARAM_NAME) == PARAM_NAME then
-        return false
-    end
     return true
 end
 
@@ -1673,6 +1705,8 @@ function m.getNoders(source)
             backward  = {},
             binfo     = {},
             backwards = {},
+            extend    = {},
+            extends   = {},
             call      = {},
             require   = {},
             skip      = {},
@@ -1703,7 +1737,7 @@ function m.compileAllNodes(source)
     end
     root._initedNoders = true
     if not root._compiledGlobals then
-        collector.dropUri(guide.getUri(root))
+        collector:dropUri(guide.getUri(root))
     end
     root._compiledGlobals = true
     --log.debug('compileNodes:', guide.getUri(root))
@@ -1842,7 +1876,7 @@ function m.compileGlobalNodes(root)
         return
     end
     if not root._compiledGlobals then
-        collector.dropUri(guide.getUri(root))
+        collector:dropUri(guide.getUri(root))
     end
     root._compiledGlobals = true
     local noders = m.getNoders(root)
@@ -1860,16 +1894,22 @@ function m.compileGlobalNodes(root)
     end)
 end
 
+for uri in files.eachFile() do
+    local state = files.getState(uri)
+    if state then
+        m.compileGlobalNodes(state.ast)
+    end
+end
+
 files.watch(function (ev, uri)
     if ev == 'update' then
         local state = files.getState(uri)
         if state then
-            --m.compileAllNodes(state.ast)
             m.compileGlobalNodes(state.ast)
         end
     end
     if ev == 'remove' then
-        collector.dropUri(uri)
+        collector:dropUri(uri)
     end
 end)
 
